@@ -191,9 +191,8 @@ class UndercloudConfig(StandaloneConfig):
                             'this certificate will be used in place of the '
                             'value for undercloud_service_certificate.  The '
                             'resulting certificate will be written to '
-                            '/etc/pki/tls/certs/'
-                            'undercloud-[undercloud_public_host].pem.  This '
-                            'certificate is signed by CA selected by the '
+                            '/etc/pki/tls/private/overcloud_endpoint.pem. This'
+                            ' certificate is signed by CA selected by the '
                             '"certificate_generation_ca" option.')
                         ),
             cfg.StrOpt('certificate_generation_ca',
@@ -284,6 +283,10 @@ class UndercloudConfig(StandaloneConfig):
                         default=True,
                         help=_('Enable or disable SELinux during the '
                                'deployment.')),
+            cfg.BoolOpt('undercloud_enable_paunch',
+                        default=True,
+                        help=_('Enable or disable Paunch to manage '
+                               'containers.')),
             cfg.BoolOpt('undercloud_update_packages',
                         default=False,
                         help=_(
@@ -359,6 +362,24 @@ class UndercloudConfig(StandaloneConfig):
                        help=(_('IPv6 address configuration mode for the '
                                'undercloud provisioning network.'))
                        ),
+            cfg.ListOpt('ironic_enabled_network_interfaces',
+                        default=['flat'],
+                        help=(_('Enabled ironic network interface '
+                                'implementations. Each hardware type must '
+                                'have at least one valid implementation '
+                                'enabled.'))
+                        ),
+            cfg.StrOpt('ironic_default_network_interface',
+                       default='flat',
+                       choices=[
+                           ('flat', 'Use one flat provider network.'),
+                           ('neutron', 'Ironic interacts with Neutron to '
+                                       'enable other network types and '
+                                       'advanced networking features.')
+                       ],
+                       help=(_('Ironic network interface implementation to '
+                               'use by default.'))
+                       ),
         ]
         return self.sort_opts(_base_opts + _opts)
 
