@@ -494,18 +494,18 @@ class Deploy(command.Command):
             'DeployedServerPortMap': {
                 ('%s-ctlplane' % hostname): {
                     'fixed_ips': [{'ip_address': ip_addr}],
-                    'subnets': [{'cidr': str(ip_nw)}],
-                    'network': {'tags': [str(ip_nw)]}
+                    'subnets': [{'cidr': str(ip_nw.cidr)}],
+                    'network': {'tags': [str(ip_nw.cidr)]}
                 },
                 'control_virtual_ip': {
                     'fixed_ips': [{'ip_address': ctlplane_vip_addr}],
-                    'subnets': [{'cidr': str(ip_nw)}],
-                    'network': {'tags': [str(ip_nw)]}
+                    'subnets': [{'cidr': str(ip_nw.cidr)}],
+                    'network': {'tags': [str(ip_nw.cidr)]}
                 },
                 'public_virtual_ip': {
                     'fixed_ips': [{'ip_address': public_vip_addr}],
-                    'subnets': [{'cidr': str(ip_nw)}],
-                    'network': {'tags': [str(ip_nw)]}
+                    'subnets': [{'cidr': str(ip_nw.cidr)}],
+                    'network': {'tags': [str(ip_nw.cidr)]}
                 }
             }
         }
@@ -731,6 +731,7 @@ class Deploy(command.Command):
         with open(stack_vstate_dropin, 'w') as dropin_file:
             yaml.safe_dump(
                 {'parameter_defaults': {
+                    'RootStackName': parsed_args.stack.lower(),
                     'StackAction': self.stack_action,
                     'DeployIdentifier': int(time.time())}},
                 dropin_file, default_flow_style=False)
@@ -824,8 +825,13 @@ class Deploy(command.Command):
 
         self.log.warning(_("** Performing Heat stack create.. **"))
         stack = orchestration_client.stacks.create(**stack_args)
-        stack_id = stack['stack']['id']
+        if not stack:
+            msg = _('The ephemeral Heat stack could not be created, please '
+                    'check logs in /var/log/heat-launcher and/or any '
+                    'possible misconfiguration.')
+            raise exceptions.DeploymentError(msg)
 
+        stack_id = stack['stack']['id']
         return "%s/%s" % (stack_name, stack_id)
 
     def _download_ansible_playbooks(self, client, stack_name,
@@ -1104,6 +1110,14 @@ class Deploy(command.Command):
                    'deployed services are running right after their '
                    'activation. Defaults to False.')
         )
+        parser.add_argument(
+            '--ansible-forks',
+            action='store',
+            default=None,
+            type=int,
+            help=_('The number of Ansible forks to use for the'
+                   ' config-download ansible-playbook command.')
+        )
 
         stack_action_group = parser.add_mutually_exclusive_group()
 
@@ -1265,7 +1279,7 @@ class Deploy(command.Command):
 
             # Wait for complete..
             status = utils.wait_for_stack_ready(orchestration_client, stack_id,
-                                                nested_depth=6)
+                                                verbose=True, nested_depth=6)
             if not status:
                 message = _("Stack create failed")
                 self.log.error(message)
@@ -1303,8 +1317,13 @@ class Deploy(command.Command):
                 shutil.copy(ansible_config, self.ansible_dir)
 
             extra_args = []
+
             if not parsed_args.inflight:
                 extra_args = ['--skip-tags', 'opendev-validation']
+
+            if parsed_args.ansible_forks:
+                extra_args.append('--forks', str(parsed_args.ansible_forks))
+
             # Kill heat, we're done with it now.
             if not parsed_args.keep_running:
                 self._kill_heat(parsed_args)
@@ -1378,6 +1397,7 @@ class Deploy(command.Command):
             else:
                 # We only get here if no errors
                 if parsed_args.output_only:
+                    rc = 0
                     success_messaging = OUTPUT_ONLY_COMPLETION_MESSAGE
                 else:
                     success_messaging = DEPLOY_COMPLETION_MESSAGE
@@ -1415,28 +1435,6 @@ class Deploy(command.Command):
     def take_action(self, parsed_args):
         self.log.debug("take_action(%s)" % parsed_args)
         utils.ansible_symlink()
-        unconf_msg = _('User did not confirm upgrade, so exiting. '
-                       'Consider using the --yes parameter if you '
-                       'prefer to skip this warning in the future')
-        try:
-            if parsed_args.upgrade and (
-                    not parsed_args.yes and sys.stdin.isatty()):
-                prompt_response = six.moves.input(
-                    ('It is strongly recommended to perform a backup '
-                     'before the upgrade. Are you sure you want to '
-                     'upgrade [y/N]?')
-                ).lower()
-                if not prompt_response.startswith('y'):
-                    raise exceptions.UndercloudUpgradeNotConfirmed(unconf_msg)
-        except (KeyboardInterrupt, EOFError) as e:
-            if e.__class__ == KeyboardInterrupt:
-                # ctrl-c
-                raise exceptions.UndercloudUpgradeNotConfirmed("(ctrl-c) %s" %
-                                                               unconf_msg)
-            else:
-                # ctrl-d
-                raise exceptions.UndercloudUpgradeNotConfirmed("(ctrl-d) %s" %
-                                                               unconf_msg)
 
         try:
             if parsed_args.standalone:
@@ -1449,6 +1447,10 @@ class Deploy(command.Command):
                 self.log.error(msg)
                 raise exceptions.DeploymentError(msg)
         finally:
+            # Copy clouds.yaml from /etc/openstack so credentials can be
+            # read by the deployment user and not only root.
+            utils.copy_clouds_yaml(parsed_args.deployment_user)
+
             # send erase sequence to reset the cmdline if paunch/ansible
             # mangled some escape sequences
             utils.reset_cmdline()
