@@ -89,6 +89,7 @@ USER_HOME = os.environ.get('HOME', '')
 TELEMETRY_DOCKER_ENV_YAML = [
     'environments/services/undercloud-gnocchi.yaml',
     'environments/services/undercloud-aodh.yaml',
+    'environments/services/undercloud-panko.yaml',
     'environments/services/undercloud-ceilometer.yaml']
 
 CONF = cfg.CONF
@@ -142,6 +143,11 @@ def _process_drivers_and_hardware_types(conf, env):
         env['IronicInspectorDiscoveryDefaultDriver'] = (
             conf.discovery_default_driver)
 
+    env['IronicEnabledNetworkInterfaces'] = \
+        conf.ironic_enabled_network_interfaces
+    env['IronicDefaultNetworkInterface'] = \
+        conf.ironic_default_network_interface
+
     # In most cases power and management interfaces are called the same, so we
     # use one variable for them.
     mgmt_interfaces = {'fake', 'ipmitool'}
@@ -224,23 +230,30 @@ def _generate_inspection_subnets():
         env_dict = {}
         s = CONF.get(subnet)
         env_dict['tag'] = subnet
-        if netaddr.IPNetwork(s.cidr).version == 4:
-            env_dict['ip_range'] = s.inspection_iprange
-        if netaddr.IPNetwork(s.cidr).version == 6:
-            if CONF['ipv6_address_mode'] == 'dhcpv6-stateful':
+        try:
+            if netaddr.IPNetwork(s.cidr).version == 4:
                 env_dict['ip_range'] = s.inspection_iprange
-            if CONF['ipv6_address_mode'] == 'dhcpv6-stateless':
-                # dnsmasq(8): A static-only subnet with address all zeros may
-                # be used as a "catch-all" address to enable replies to all
-                # Information-request packets on a subnet which is provided
-                # with stateless DHCPv6, ie --dhcp-range=::,static
-                env_dict['ip_range'] = ','.join(
-                    [str(netaddr.IPNetwork(s.cidr).ip), 'static'])
-        env_dict['netmask'] = str(netaddr.IPNetwork(s.cidr).netmask)
-        env_dict['gateway'] = s.gateway
-        env_dict['host_routes'] = s.host_routes
-        env_dict['mtu'] = CONF.local_mtu
-        env_list.append(env_dict)
+            if netaddr.IPNetwork(s.cidr).version == 6:
+                if CONF['ipv6_address_mode'] == 'dhcpv6-stateful':
+                    env_dict['ip_range'] = s.inspection_iprange
+                if CONF['ipv6_address_mode'] == 'dhcpv6-stateless':
+                    # dnsmasq(8): A static-only subnet with address all zeros
+                    # may be used as a "catch-all" address to enable replies to
+                    # all Information-request packets on a subnet which is
+                    # provided with stateless DHCPv6, ie --dhcp-range=::,static
+                    env_dict['ip_range'] = ','.join(
+                        [str(netaddr.IPNetwork(s.cidr).ip), 'static'])
+            env_dict['netmask'] = str(netaddr.IPNetwork(s.cidr).netmask)
+            env_dict['gateway'] = s.gateway
+            env_dict['host_routes'] = s.host_routes
+            env_dict['mtu'] = CONF.local_mtu
+            env_list.append(env_dict)
+        except Exception as e:
+            msg = _('Invalid configuration data in subnet "{}". Double check '
+                    'the settings for this subnet. Error: {}').format(subnet,
+                                                                      e)
+            LOG.error(msg)
+            raise exceptions.DeploymentError(msg)
     return env_list
 
 
@@ -320,9 +333,9 @@ def _calculate_allocation_pools(subnet):
     ip_set.remove(netaddr.IPAddress(subnet.get('gateway')))
     ip_set.remove(netaddr.IPNetwork(CONF.local_ip).ip)
     ip_set.remove(netaddr.IPNetwork(utils.get_single_ip(
-        CONF.undercloud_admin_host)))
+        CONF.undercloud_admin_host, ip_version=ip_network.version)))
     ip_set.remove(netaddr.IPNetwork(utils.get_single_ip(
-        CONF.undercloud_public_host)))
+        CONF.undercloud_public_host, ip_version=ip_network.version)))
     # Remove dns nameservers
     for addr in subnet.get('dns_nameservers', []):
         ip_set.remove(netaddr.IPAddress(addr))
@@ -382,18 +395,21 @@ def _process_network_args(env):
     # value here.
     if (CONF.get('generate_service_certificate') or
             CONF.get('undercloud_service_certificate')):
-        undercloud_ips = [
-            CONF.local_ip.split('/')[0],
-            CONF.undercloud_admin_host,
-            CONF.undercloud_public_host
-        ]
-        if len(undercloud_ips) != len(set(undercloud_ips)):
-            msg = ("The same IP is used for multiple endpoints. Please use "
-                   "unique ips for local_ip, undercloud_admin_host and "
-                   "undercloud_public_host")
+        if CONF.local_ip.split('/')[0] == CONF.undercloud_admin_host:
+            msg = ("Different IPs should be assigned to local_ip and "
+                   "undercloud_admin_host")
             raise exceptions.InvalidConfiguration(msg)
 
 
+def _process_chrony_acls(env):
+    """Populate ACL rules for chrony to allow ctlplane subnets"""
+    acl_rules = []
+    for subnet in CONF.subnets:
+        s = CONF.get(subnet)
+        acl_rules.append('allow ' + s.get('cidr'))
+    env['ChronyAclRules'] = acl_rules
+
+
 def prepare_undercloud_deploy(upgrade=False, no_validations=True,
                               verbose_level=1, yes=False,
                               force_stack_update=False, dry_run=False,
@@ -441,6 +457,9 @@ def prepare_undercloud_deploy(upgrade=False, no_validations=True,
     # Set up parameters for undercloud networking
     _process_network_args(env_data)
 
+    # Setup parameter for Chrony ACL rules
+    _process_chrony_acls(env_data)
+
     # Parse the undercloud.conf options to include necessary args and
     # yaml files for undercloud deploy command
 
@@ -449,6 +468,11 @@ def prepare_undercloud_deploy(upgrade=False, no_validations=True,
     else:
         env_data['SELinuxMode'] = 'permissive'
 
+    if CONF.get('undercloud_enable_paunch'):
+        env_data['EnablePaunch'] = True
+    else:
+        env_data['EnablePaunch'] = False
+
     if CONF.get('undercloud_ntp_servers', None):
         env_data['NtpServer'] = CONF['undercloud_ntp_servers']
 
@@ -591,6 +615,10 @@ def prepare_undercloud_deploy(upgrade=False, no_validations=True,
         deploy_args += ['-e', os.path.join(
             tht_templates, "environments/services/novajoin.yaml")]
         env_data['NovajoinIpaOtp'] = CONF['ipa_otp']
+    elif CONF.get('ipa_otp'):
+        deploy_args += ['-e', os.path.join(
+            tht_templates, "environments/services/undercloud-tls.yaml")]
+        env_data['UndercloudIpaOtp'] = CONF['ipa_otp']
 
     if CONF.get('enable_zaqar'):
         deploy_args += ['-e', os.path.join(
@@ -641,10 +669,13 @@ def prepare_undercloud_deploy(upgrade=False, no_validations=True,
 
     if (CONF.get('generate_service_certificate') or
             CONF.get('undercloud_service_certificate')):
+        local_net = netaddr.IPNetwork(CONF.get('local_ip'))
+
         endpoint_environment = _get_tls_endpoint_environment(
             CONF.get('undercloud_public_host'), tht_templates)
 
-        public_host = utils.get_single_ip(CONF.get('undercloud_public_host'))
+        public_host = utils.get_single_ip(CONF.get('undercloud_public_host'),
+                                          ip_version=local_net.version)
         public_ip = netaddr.IPAddress(public_host)
         deploy_args += ['--public-virtual-ip', public_host]
 
@@ -653,11 +684,11 @@ def prepare_undercloud_deploy(upgrade=False, no_validations=True,
             extra_host = public_host + ' ' + CONF.get('undercloud_public_host')
             env_data['ExtraHostFileEntries'] = extra_host
 
-        admin_host = utils.get_single_ip(CONF.get('undercloud_admin_host'))
+        admin_host = utils.get_single_ip(CONF.get('undercloud_admin_host'),
+                                         ip_version=local_net.version)
         admin_ip = netaddr.IPAddress(admin_host)
         deploy_args += ['--control-virtual-ip', admin_host]
 
-        local_net = netaddr.IPNetwork(CONF.get('local_ip'))
         if CONF.get('net_config_override', None):
             if (admin_ip not in local_net.cidr):
                 LOG.warning('You may need to specify a custom '
